@@ -30,7 +30,10 @@ drawDetails.dlgrob <- function(x,recording){
   cm.data <- unique(cm.data)
   gpargs <- c("cex","alpha","fontface","fontfamily","col")
   gp <- do.call(gpar,cm.data[names(cm.data)%in%gpargs])
-  if(x$debug)print(cm.data)
+  if(x$debug){
+    print(cm.data)
+    ##browser()
+  }
   with(cm.data,{
     grid.text(groups,x,y,hjust=hjust,vjust=vjust,rot=rot,default.units="cm",
               gp=gp)
@@ -48,24 +51,38 @@ dlgrob <- function
  ...
  ){
   grob(data=data,method=method,debug=debug,axes2native=axes2native,
-       cl="dlgrob",...)
+       cl="dlgrob",
+       name=if(is.character(method)){
+         sprintf("GRID.dlgrob.%s",method[1])
+       }else{
+         NULL
+       },...)
 }
 
-direct.label <- structure(function
-### Add direct labels to a plot. This is a S3 generic and there are
-### appropriate methods for "trellis" and "ggplot" objects.
+direct.label <- structure(function # Direct labels for color decoding
+### Add direct labels to a plot, and hide the color legend. Modern
+### plotting packages like lattice and ggplot2 show automatic legends
+### based on the variable specified for color, but these legends can
+### be confusing if there are too many colors. Direct labels are a
+### useful and clear alternative to a confusing legend in many common
+### plots.
 (p,
-### The plot to which you would like to add direct labels.
+### The "lattice" or "ggplot" object with things drawn in different
+### colors.
  method=NULL,
-### Positioning Method.
+### Positioning Method, which determines the positions of the direct
+### labels as a function of the plotted data. If NULL, we examine the
+### plot p and try to choose an appropriate default. See ?apply.method
+### for more information about Positioning Methods.
  debug=FALSE
 ### Show debug output?
  ){
+  ##alias<< directlabels
   if(is.character(method)&&method[1]=="legend")
     UseMethod("uselegend")
   else
     UseMethod("direct.label")
-### The plot object, with direct labels added.
+### A plot with direct labels and no color legend.
 },ex=function(){
   library(ggplot2)
   ## direct label simple ggplot2 scatterplot
@@ -119,6 +136,7 @@ direct.label <- structure(function
     plot(direct.label(densityplot(~gcsescore|gender,Chem97,plot.points=FALSE,
                                   groups=factor(score),layout=c(1,2),n=500)))
   }
+  require(reshape2)
   iris2 <- melt(iris,id="Species")
   direct.label(densityplot(~value|variable,iris2,groups=Species,scales="free"))
   loci <- data.frame(ppp=c(rbeta(800,10,10),rbeta(100,0.15,1),rbeta(100,1,0.15)),
@@ -129,7 +147,7 @@ direct.label <- structure(function
   direct.label(lplot,static.labels(c(0,0.5,1),0,c("POS","NEU","BAL"),vjust=1.1))
   ## respect the manual color scale. these 2 should be the same:
   lplot2 <- direct.label(lplot)+
-    scale_colour_manual(values=c("red","black","blue"),legend=FALSE)
+    scale_colour_manual(values=c("red","black","blue"),guide="none")
   print(lplot2)
   print(direct.label(lplot+scale_colour_manual(values=c("red","black","blue"))))
   
@@ -203,6 +221,7 @@ direct.label <- structure(function
     ## make the x axis degrees of freedom
     df <- sapply(lambda,function(l)sum(dsq/(dsq+l)))
     D <- data.frame(t(fit$coef),lambda,df) # scaled coefs
+    require(reshape2)
     molt <- melt(D,id=c("lambda","df"))
     ## add in the points for df=0
     limpts <- transform(subset(molt,lambda==0),lambda=Inf,df=0,value=0)
@@ -223,11 +242,12 @@ direct.label <- structure(function
                 sub="grey line shows coefficients chosen by cross-validation",
                 xlab=expression(df(lambda)))
     print(direct.label(update(p,xlim=c(0,9.25)),
-                       list(last.qp,cex=0.75,dl.trans(x=x+0.1))))
+                       list(cex=1.5,last.qp,dl.trans(x=x+0.1))))
   }  
   ## some data from clustering algorithms
   data(iris.l1.cluster,package="directlabels")
-  p <- ggplot(iris.l1.cluster,aes(lambda,alpha,group=row,colour=Species))+
+  ilc <- transform(iris.l1.cluster,coefficient=alpha)
+  p <- ggplot(ilc,aes(lambda,coefficient,group=row,colour=Species))+
     geom_line(alpha=1/4)+
     facet_grid(col~.)
   p2 <- p+xlim(-0.0025,max(iris.l1.cluster$lambda))
@@ -243,7 +263,7 @@ direct.label <- structure(function
   print(direct.label(p,"extreme.grid"))
   ## respect the color scale. these should look the same:
   print(direct.label(p+scale_colour_manual(values=rainbow(8))))
-  print(direct.label(p)+scale_colour_manual(values=rainbow(8),legend=FALSE))
+  print(direct.label(p)+scale_colour_manual(values=rainbow(8),guide="none"))
 
   lattice.options(oldopt)
 })
